@@ -2,7 +2,7 @@ import xcffib
 import struct
 import six
 MAJOR_VERSION = 1
-MINOR_VERSION = 4
+MINOR_VERSION = 6
 key = xcffib.ExtensionKey("RANDR")
 _events = {}
 _errors = {}
@@ -149,6 +149,7 @@ class NotifyMask:
     ProviderChange = 1 << 4
     ProviderProperty = 1 << 5
     ResourceChange = 1 << 6
+    Lease = 1 << 7
 class GetScreenInfoReply(xcffib.Reply):
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
@@ -368,29 +369,6 @@ class Transform:
     ScaleUp = 1 << 1
     ScaleDown = 1 << 2
     Projective = 1 << 3
-class GetCrtcTransformReply(xcffib.Reply):
-    def __init__(self, unpacker):
-        if isinstance(unpacker, xcffib.Protobj):
-            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
-        xcffib.Reply.__init__(self, unpacker)
-        base = unpacker.offset
-        unpacker.unpack("xx2x4x")
-        self.pending_transform = render.TRANSFORM(unpacker)
-        self.has_transforms, = unpacker.unpack("B3x")
-        unpacker.pad(render.TRANSFORM)
-        self.current_transform = render.TRANSFORM(unpacker)
-        self.pending_len, self.pending_nparams, self.current_len, self.current_nparams = unpacker.unpack("4xHHHH")
-        unpacker.pad("c")
-        self.pending_filter_name = xcffib.List(unpacker, "c", self.pending_len)
-        unpacker.pad("i")
-        self.pending_params = xcffib.List(unpacker, "i", self.pending_nparams)
-        unpacker.pad("c")
-        self.current_filter_name = xcffib.List(unpacker, "c", self.current_len)
-        unpacker.pad("i")
-        self.current_params = xcffib.List(unpacker, "i", self.current_nparams)
-        self.bufsize = unpacker.offset - base
-class GetCrtcTransformCookie(xcffib.Cookie):
-    reply_type = GetCrtcTransformReply
 class GetPanningReply(xcffib.Reply):
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
@@ -528,6 +506,7 @@ class Notify:
     ProviderChange = 3
     ProviderProperty = 4
     ResourceChange = 5
+    Lease = 6
 class CrtcChange(xcffib.Struct):
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
@@ -663,6 +642,77 @@ class ResourceChange(xcffib.Struct):
         self.timestamp = timestamp
         self.window = window
         return self
+class MonitorInfo(xcffib.Struct):
+    def __init__(self, unpacker):
+        if isinstance(unpacker, xcffib.Protobj):
+            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
+        xcffib.Struct.__init__(self, unpacker)
+        base = unpacker.offset
+        self.name, self.primary, self.automatic, self.nOutput, self.x, self.y, self.width, self.height, self.width_in_millimeters, self.height_in_millimeters = unpacker.unpack("IBBHhhHHII")
+        self.outputs = xcffib.List(unpacker, "I", self.nOutput)
+        self.bufsize = unpacker.offset - base
+    def pack(self):
+        buf = six.BytesIO()
+        buf.write(struct.pack("=IBBHhhHHII", self.name, self.primary, self.automatic, self.nOutput, self.x, self.y, self.width, self.height, self.width_in_millimeters, self.height_in_millimeters))
+        buf.write(xcffib.pack_list(self.outputs, "I"))
+        return buf.getvalue()
+    @classmethod
+    def synthetic(cls, name, primary, automatic, nOutput, x, y, width, height, width_in_millimeters, height_in_millimeters, outputs):
+        self = cls.__new__(cls)
+        self.name = name
+        self.primary = primary
+        self.automatic = automatic
+        self.nOutput = nOutput
+        self.x = x
+        self.y = y
+        self.width = width
+        self.height = height
+        self.width_in_millimeters = width_in_millimeters
+        self.height_in_millimeters = height_in_millimeters
+        self.outputs = outputs
+        return self
+class GetMonitorsReply(xcffib.Reply):
+    def __init__(self, unpacker):
+        if isinstance(unpacker, xcffib.Protobj):
+            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
+        xcffib.Reply.__init__(self, unpacker)
+        base = unpacker.offset
+        self.timestamp, self.nMonitors, self.nOutputs = unpacker.unpack("xx2x4xIII12x")
+        self.monitors = xcffib.List(unpacker, MonitorInfo, self.nMonitors)
+        self.bufsize = unpacker.offset - base
+class GetMonitorsCookie(xcffib.Cookie):
+    reply_type = GetMonitorsReply
+class CreateLeaseReply(xcffib.Reply):
+    def __init__(self, unpacker):
+        if isinstance(unpacker, xcffib.Protobj):
+            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
+        xcffib.Reply.__init__(self, unpacker)
+        base = unpacker.offset
+        self.nfd, = unpacker.unpack("xB2x4x24x")
+        self.bufsize = unpacker.offset - base
+class CreateLeaseCookie(xcffib.Cookie):
+    reply_type = CreateLeaseReply
+class LeaseNotify(xcffib.Struct):
+    def __init__(self, unpacker):
+        if isinstance(unpacker, xcffib.Protobj):
+            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
+        xcffib.Struct.__init__(self, unpacker)
+        base = unpacker.offset
+        self.timestamp, self.window, self.lease, self.created = unpacker.unpack("IIIB15x")
+        self.bufsize = unpacker.offset - base
+    def pack(self):
+        buf = six.BytesIO()
+        buf.write(struct.pack("=IIIB15x", self.timestamp, self.window, self.lease, self.created))
+        return buf.getvalue()
+    fixed_size = 28
+    @classmethod
+    def synthetic(cls, timestamp, window, lease, created):
+        self = cls.__new__(cls)
+        self.timestamp = timestamp
+        self.window = window
+        self.lease = lease
+        self.created = created
+        return self
 class NotifyData(xcffib.Union):
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
@@ -674,6 +724,7 @@ class NotifyData(xcffib.Union):
         self.pc = ProviderChange(unpacker.copy())
         self.pp = ProviderProperty(unpacker.copy())
         self.rc = ResourceChange(unpacker.copy())
+        self.lc = LeaseNotify(unpacker.copy())
     def pack(self):
         buf = six.BytesIO()
         buf.write(self.cc.pack() if hasattr(self.cc, "pack") else CrtcChange.synthetic(*self.cc).pack())
@@ -815,10 +866,10 @@ class randrExtension(xcffib.Extension):
         buf.write(xcffib.pack_list(filter_name, "c"))
         buf.write(xcffib.pack_list(filter_params, "i"))
         return self.send_request(26, buf, is_checked=is_checked)
-    def GetCrtcTransform(self, crtc, is_checked=True):
+    def GetCrtcTransform(self, crtc, is_checked=False):
         buf = six.BytesIO()
         buf.write(struct.pack("=xx2xI", crtc))
-        return self.send_request(27, buf, GetCrtcTransformCookie, is_checked=is_checked)
+        return self.send_request(27, buf, is_checked=is_checked)
     def GetPanning(self, crtc, is_checked=True):
         buf = six.BytesIO()
         buf.write(struct.pack("=xx2xI", crtc))
@@ -877,4 +928,27 @@ class randrExtension(xcffib.Extension):
         buf = six.BytesIO()
         buf.write(struct.pack("=xx2xIIIIIBB2x", provider, property, type, long_offset, long_length, delete, pending))
         return self.send_request(41, buf, GetProviderPropertyCookie, is_checked=is_checked)
+    def GetMonitors(self, window, get_active, is_checked=True):
+        buf = six.BytesIO()
+        buf.write(struct.pack("=xx2xIB", window, get_active))
+        return self.send_request(42, buf, GetMonitorsCookie, is_checked=is_checked)
+    def SetMonitor(self, window, monitorinfo, is_checked=False):
+        buf = six.BytesIO()
+        buf.write(struct.pack("=xx2xI", window))
+        buf.write(monitorinfo.pack() if hasattr(monitorinfo, "pack") else MonitorInfo.synthetic(*monitorinfo).pack())
+        return self.send_request(43, buf, is_checked=is_checked)
+    def DeleteMonitor(self, window, name, is_checked=False):
+        buf = six.BytesIO()
+        buf.write(struct.pack("=xx2xII", window, name))
+        return self.send_request(44, buf, is_checked=is_checked)
+    def CreateLease(self, window, lid, num_crtcs, num_outputs, crtcs, outputs, is_checked=True):
+        buf = six.BytesIO()
+        buf.write(struct.pack("=xx2xIIHH", window, lid, num_crtcs, num_outputs))
+        buf.write(xcffib.pack_list(crtcs, "I"))
+        buf.write(xcffib.pack_list(outputs, "I"))
+        return self.send_request(45, buf, CreateLeaseCookie, is_checked=is_checked)
+    def FreeLease(self, lid, terminate, is_checked=False):
+        buf = six.BytesIO()
+        buf.write(struct.pack("=xx2xIB", lid, terminate))
+        return self.send_request(46, buf, is_checked=is_checked)
 xcffib._add_ext(key, randrExtension, _events, _errors)
