@@ -134,6 +134,6 @@ class screensaverExtension(xcffib.Extension):
         return self.send_request(4, buf, is_checked=is_checked)
     def Suspend(self, suspend, is_checked=False):
         buf = six.BytesIO()
-        buf.write(struct.pack("=xx2xB3x", suspend))
+        buf.write(struct.pack("=xx2xI", suspend))
         return self.send_request(5, buf, is_checked=is_checked)
 xcffib._add_ext(key, screensaverExtension, _events, _errors)
